@@ -16,6 +16,8 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 
+#include <simgear_config.h>
+
 #include "CanvasElement.hxx"
 #include <simgear/canvas/Canvas.hxx>
 #include <simgear/canvas/CanvasEventVisitor.hxx>
@@ -640,6 +642,27 @@ namespace canvas
       _scissor->_coord_reference = rf;
   }
 
+    //----------------------------------------------------------------------------
+    void Element::setRotation(unsigned int index, double r)
+    {
+        _node->getChild(NAME_TRANSFORM, index, true)->setDoubleValue("rot", r);
+    }
+
+    //----------------------------------------------------------------------------
+    void Element::setTranslation(unsigned int index, double x, double y)
+    {
+        SGPropertyNode* tf = _node->getChild(NAME_TRANSFORM, index, true);
+        tf->getChild("t", 0, true)->setDoubleValue(x);
+        tf->getChild("t", 1, true)->setDoubleValue(y);
+    }
+
+    //----------------------------------------------------------------------------
+    void Element::setTransformEnabled(unsigned int index, bool enabled)
+    {
+        SGPropertyNode* tf = _node->getChild(NAME_TRANSFORM, index, true);
+        tf->setBoolValue("enabled", enabled);
+    }
+
   //----------------------------------------------------------------------------
   osg::BoundingBox Element::getBoundingBox() const
   {
@@ -701,6 +724,9 @@ namespace canvas
         continue;
 
       SGPropertyNode* tf_node = _node->getChild("tf", i, true);
+      if (!tf_node->getBoolValue("enabled", true)) {
+        continue; // skip disabled transforms
+      }
 
       // Build up the matrix representation of the current transform node
       osg::Matrix tf;
