@@ -16,6 +16,8 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 
+#include <simgear_config.h>
+
 #include "CanvasImage.hxx"
 
 #include <simgear/canvas/Canvas.hxx>
@@ -487,11 +489,17 @@ namespace canvas
     if(    !fill.empty() // If no color is given default to white
         && !parseColor(fill, color) )
       return;
-
-    _colors->front() = color;
-    _colors->dirty();
+    setFill(color);
   }
 
+    //----------------------------------------------------------------------------
+    void Image::setFill(const osg::Vec4& color)
+    {
+        _colors->front() = color;
+        _colors->dirty();
+    }
+
+
   //----------------------------------------------------------------------------
   void Image::setOutset(const std::string& outset)
   {
@@ -506,6 +514,13 @@ namespace canvas
     _attributes_dirty |= SRC_RECT;
   }
 
+  //----------------------------------------------------------------------------
+  void Image::setSourceRect(const SGRect<float>& sourceRect)
+  {
+      _attributes_dirty |= SRC_RECT;
+      _src_rect = sourceRect;
+  }
+
   //----------------------------------------------------------------------------
   void Image::setSlice(const std::string& slice)
   {
