@@ -16,14 +16,18 @@
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 
+#include <simgear_config.h>
+
 #include "CanvasPath.hxx"
 #include <simgear/scene/util/parse_color.hxx>
+#include <simgear/misc/strutils.hxx>
 
 #include <osg/Drawable>
 #include <osg/Version>
 
 #include <vg/openvg.h>
 #include <cassert>
+#include <cctype>
 
 namespace simgear
 {
@@ -57,6 +61,153 @@ namespace canvas
    */
   std::vector<float> splitAndConvert(const char del[], const std::string& str);
 
+    static float parseCSSNumber(const std::string& s)
+    {
+        if (strutils::ends_with(s, "px")) {
+            return std::stof(s.substr(0, s.length() - 2));
+        } else if (s.back() == '%') {
+            float f = std::stof(s.substr(0, s.length() - 1));
+            return f / 100.0f;
+        }
+        return std::stof(s);
+    }
+
+    //----------------------------------------------------------------------------
+    static bool parseSVGPathToVGPath(const std::string& svgPath, CmdList& commands, CoordList& coords)
+    {
+        const string_list& tokens = simgear::strutils::split_on_any_of(svgPath, "\t \n\r,");
+        char activeSVGCommand = 0;
+        bool isRelative = false;
+        int tokensNeeded = 0;
+
+        for (auto it = tokens.begin(); it != tokens.end(); ) {
+            // set up the new command data
+            if ((it->size() == 1) && std::isalpha(it->at(0))) {
+                const char svgCommand = std::toupper(it->at(0));
+                isRelative = std::islower(it->at(0));
+                switch (svgCommand) {
+                case 'Z':
+                        tokensNeeded = 0;
+                        break;
+                case 'M':
+                case 'L':
+                case 'T':
+                        tokensNeeded = 2;
+                        break;
+                case 'H':
+                case 'V':
+                        tokensNeeded = 1;
+                        break;
+                case 'C':
+                        tokensNeeded = 6;
+                        break;
+                case 'S':
+                case 'Q':
+                        tokensNeeded = 4;
+                        break;
+                case 'A':
+                        tokensNeeded = 7;
+                        break;
+                default:
+                    SG_LOG(SG_GENERAL, SG_WARN, "unrecognized SVG path command: "
+                           << *it << " at token " <<  std::distance(tokens.begin(), it));
+                    return false;
+                }
+
+                activeSVGCommand = svgCommand;
+                ++it; // advance to first coordinate token
+            }
+
+            const int numTokensRemaining = std::distance(it, tokens.end());
+            if (numTokensRemaining < tokensNeeded) {
+                SG_LOG(SG_GENERAL, SG_WARN, "insufficent SVG path tokens");
+                return false;
+            }
+
+            bool pushTokensDirectly = true;
+            if (activeSVGCommand == 'Z') {
+                commands.push_back(VG_CLOSE_PATH);
+                activeSVGCommand = 0;
+            } else if (activeSVGCommand == 'M') {
+                commands.push_back(VG_MOVE_TO | isRelative);
+                activeSVGCommand = 'L';
+            } else if (activeSVGCommand == 'L') {
+                commands.push_back(VG_LINE_TO | isRelative);
+            } else if (activeSVGCommand == 'H') {
+                commands.push_back(VG_HLINE_TO | isRelative);
+            } else if (activeSVGCommand == 'V') {
+                commands.push_back(VG_HLINE_TO | isRelative);
+            } else if (activeSVGCommand == 'C') {
+                commands.push_back(VG_CUBIC_TO | isRelative);
+            } else if (activeSVGCommand == 'S') {
+                commands.push_back(VG_SCUBIC_TO | isRelative);
+            } else if (activeSVGCommand == 'Q') {
+                commands.push_back(VG_SCUBIC_TO | isRelative);
+            } else if (activeSVGCommand == 'T') {
+                commands.push_back(VG_SCUBIC_TO | isRelative);
+            } else if (activeSVGCommand == 'A') {
+                pushTokensDirectly = false; // deal with tokens manually
+                coords.push_back(parseCSSNumber(*it++)); // rx
+                coords.push_back(parseCSSNumber(*it++)); // ry
+                coords.push_back(parseCSSNumber(*it++)); // x-axis rotation
+
+                const bool isLargeArc = std::stoi(*it++); // large-angle
+                const bool isCCW = std::stoi(*it++); // sweep-flag
+
+                int vgCmd = isLargeArc ? (isCCW ? VG_LCCWARC_TO : VG_LCWARC_TO) :
+                    (isCCW ? VG_SCCWARC_TO : VG_SCWARC_TO);
+
+                coords.push_back(parseCSSNumber(*it++));
+                coords.push_back(parseCSSNumber(*it++));
+                commands.push_back(vgCmd | isRelative);
+            } else {
+                SG_LOG(SG_GENERAL, SG_WARN, "malformed SVG path string: expected a command at token:"
+                       << std::distance(tokens.begin(), it) << " :" << *it);
+                return false;
+            }
+
+            if (pushTokensDirectly) {
+                for (int i=0; i<tokensNeeded;++i) {
+                    coords.push_back(parseCSSNumber(*it++));
+                }
+            }
+        } // of tokens iteration
+
+        return true;
+    }
+
+
+    //---------------------------------------------------------------------------
+
+    static SGVec2f parseRectCornerRadius(SGPropertyNode* node, const std::string& xDir, const std::string& yDir, bool& haveCorner)
+    {
+        haveCorner = false;
+        std::string propName = "border-" + yDir + "-" + xDir + "-radius";
+        if (!node->hasChild(propName)) {
+            propName = "border-" + yDir + "-radius";
+            if (!node->hasChild(propName)) {
+                propName = "border-radius";
+            }
+        }
+
+        PropertyList props = node->getChildren(propName);
+        if (props.size() == 1) {
+            double r = props.at(0)->getDoubleValue(propName);
+            haveCorner = true;
+            return SGVec2f(r, r);
+        }
+
+        if (props.size() >= 2 ) {
+            haveCorner = true;
+            return SGVec2f(props.at(0)->getDoubleValue(),
+                           props.at(1)->getDoubleValue());
+        }
+
+        return SGVec2f(-1.0f, -1.0f);
+    }
+
+    //----------------------------------------------------------------------------
+
   class Path::PathDrawable:
     public osg::Drawable
   {
@@ -533,7 +684,9 @@ namespace canvas
               const Style& parent_style,
               ElementWeakPtr parent ):
     Element(canvas, node, parent_style, parent),
-    _path( new PathDrawable(this) )
+    _path( new PathDrawable(this) ),
+    _hasSVG(false),
+    _hasRect(false)
   {
     staticInit();
 
@@ -561,6 +714,23 @@ namespace canvas
       _attributes_dirty &= ~(CMDS | COORDS);
     }
 
+    // SVG path overrides manual cmd/coord specification
+    if ( _hasSVG && (_attributes_dirty & SVG))
+    {
+        CmdList cmds;
+        CoordList coords;
+        parseSVGPathToVGPath(_node->getStringValue("svg"), cmds, coords);
+        _path->setSegments(cmds, coords);
+        _attributes_dirty &= ~SVG;
+    }
+
+    if ( _hasRect &&(_attributes_dirty & RECT))
+    {
+        parseRectToVGPath();
+        _attributes_dirty &= ~RECT;
+
+    }
+
     Element::update(dt);
   }
 
@@ -630,16 +800,75 @@ namespace canvas
     childChanged(child);
   }
 
+  //----------------------------------------------------------------------------
+  void Path::setSVGPath(const std::string& svgPath)
+  {
+    _node->setStringValue("svg", svgPath);
+    _hasSVG = true;
+    _attributes_dirty |= SVG;
+  }
+
+  //----------------------------------------------------------------------------
+  void Path::setRect(const SGRect<float> &r)
+  {
+    _rect = r;
+    _hasRect = true;
+    _attributes_dirty |= RECT;
+  }
+
+  //----------------------------------------------------------------------------
+  void Path::setRoundRect(const SGRect<float> &r, float radiusX, float radiusY)
+  {
+    if (radiusY < 0.0) {
+        radiusY = radiusX;
+    }
+
+    setRect(r);
+    _node->getChild("border-radius", 0, true)->setDoubleValue(radiusX);
+    _node->getChild("border-radius", 1, true)->setDoubleValue(radiusY);
+  }
+
   //----------------------------------------------------------------------------
   void Path::childChanged(SGPropertyNode* child)
   {
+      const std::string& name = child->getNameString();
+      const std::string &prName = child->getParent()->getNameString();
+
+      if (simgear::strutils::starts_with(name, "border-"))
+      {
+          _attributes_dirty |= RECT;
+          return;
+      }
+
+      if (prName == "rect") {
+          _hasRect = true;
+          if (name == "left") {
+              _rect.setLeft(child->getDoubleValue());
+          } else if (name == "top") {
+              _rect.setTop(child->getDoubleValue());
+          } else if (name == "right") {
+              _rect.setRight(child->getDoubleValue());
+          } else if (name == "bottom") {
+              _rect.setBottom(child->getDoubleValue());
+          } else if (name == "width") {
+              _rect.setWidth(child->getDoubleValue());
+          } else if (name == "height") {
+              _rect.setHeight(child->getDoubleValue());
+          }
+          _attributes_dirty |= RECT;
+          return;
+      }
+
     if( child->getParent() != _node )
       return;
 
-    if( child->getNameString() == "cmd" )
+    if( name == "cmd" )
       _attributes_dirty |= CMDS;
-    else if( child->getNameString() == "coord" )
+    else if( name == "coord" )
       _attributes_dirty |= COORDS;
+    else if ( name == "svg")
+      _hasSVG = true;
+      _attributes_dirty |= SVG;
   }
 
   //----------------------------------------------------------------------------
@@ -664,5 +893,67 @@ namespace canvas
     return values;
   }
 
+    //----------------------------------------------------------------------------
+
+    void operator+=(CoordList& base, const std::initializer_list<VGfloat>& other)
+    {
+        base.insert(base.end(), other.begin(), other.end());
+    }
+
+    void Path::parseRectToVGPath()
+    {
+        CmdList commands;
+        CoordList coords;
+        commands.reserve(4);
+        coords.reserve(8);
+
+        bool haveCorner = false;
+        SGVec2f topLeft = parseRectCornerRadius(_node, "left", "top", haveCorner);
+        if (haveCorner) {
+            commands.push_back(VG_MOVE_TO_ABS);
+            coords += {_rect.l(), _rect.t() + topLeft.y()};
+            commands.push_back(VG_SCCWARC_TO_REL);
+            coords += {topLeft.x(), topLeft.y(), 0.0, topLeft.x(), -topLeft.y()};
+        } else {
+            commands.push_back(VG_MOVE_TO_ABS);
+            coords += {_rect.l(), _rect.t()};
+        }
+
+        SGVec2f topRight = parseRectCornerRadius(_node, "right", "top", haveCorner);
+        if (haveCorner) {
+            commands.push_back(VG_HLINE_TO_ABS);
+            coords += {_rect.r() - topRight.x()};
+            commands.push_back(VG_SCCWARC_TO_REL);
+            coords += {topRight.x(), topRight.y(), 0.0, topRight.x(), topRight.y()};
+        } else {
+            commands.push_back(VG_HLINE_TO_ABS);
+            coords += {_rect.r()};
+        }
+
+        SGVec2f bottomRight = parseRectCornerRadius(_node, "right", "bottom", haveCorner);
+        if (haveCorner) {
+            commands.push_back(VG_VLINE_TO_ABS);
+            coords += {_rect.b() - bottomRight.y()};
+            commands.push_back(VG_SCCWARC_TO_REL);
+            coords += {bottomRight.x(), bottomRight.y(), 0.0, -bottomRight.x(), bottomRight.y()};
+        } else {
+            commands.push_back(VG_VLINE_TO_ABS);
+            coords += {_rect.b()};
+        }
+
+        SGVec2f bottomLeft = parseRectCornerRadius(_node, "left", "bottom", haveCorner);
+        if (haveCorner) {
+            commands.push_back(VG_HLINE_TO_ABS);
+            coords += {_rect.l() + bottomLeft.x()};
+            commands.push_back(VG_SCCWARC_TO_REL);
+            coords += {bottomLeft.x(), bottomLeft.y(), 0.0, -bottomLeft.x(), -bottomLeft.y()};
+        } else {
+            commands.push_back(VG_HLINE_TO_ABS);
+            coords += {_rect.l()};
+        }
+
+        commands.push_back(VG_CLOSE_PATH);
+        _path->setSegments(commands, coords);
+    }
 } // namespace canvas
 } // namespace simgear
