@@ -36,7 +36,7 @@
 #include <simgear/threads/SGQueue.hxx>
 #include <simgear/threads/SGGuard.hxx>
 
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 #include <simgear/misc/sg_path.hxx>
 
 #if defined (SG_WINDOWS)
@@ -46,39 +46,7 @@
     #include <io.h>
 #endif
 
-const char* debugClassToString(sgDebugClass c)
-{
-    switch (c) {
-    case SG_NONE:       return "none";
-    case SG_TERRAIN:    return "terrain";
-    case SG_ASTRO:      return "astro";
-    case SG_FLIGHT:     return "flight";
-    case SG_INPUT:      return "input";
-    case SG_GL:         return "opengl";
-    case SG_VIEW:       return "view";
-    case SG_COCKPIT:    return "cockpit";
-    case SG_GENERAL:    return "general";
-    case SG_MATH:       return "math";
-    case SG_EVENT:      return "event";
-    case SG_AIRCRAFT:   return "aircraft";
-    case SG_AUTOPILOT:  return "autopilot";
-    case SG_IO:         return "io";
-    case SG_CLIPPER:    return "clipper";
-    case SG_NETWORK:    return "network";
-    case SG_ATC:        return "atc";
-    case SG_NASAL:      return "nasal";
-    case SG_INSTR:      return "instruments";
-    case SG_SYSTEMS:    return "systems";
-    case SG_AI:         return "ai";
-    case SG_ENVIRONMENT:return "environment";
-    case SG_SOUND:      return "sound";
-    case SG_NAVAID:     return "navaid";
-    case SG_GUI:        return "gui";
-    case SG_TERRASYNC:  return "terrasync";
-    case SG_PARTICLES:  return "particles";
-    default:            return "unknown";
-    }
-}
+
 
 //////////////////////////////////////////////////////////////////////////////
 
@@ -102,6 +70,40 @@ void LogCallback::setLogLevels( sgDebugClass c, sgDebugPriority p )
 	m_class = c;
 }
 
+const char* LogCallback::debugClassToString(sgDebugClass c)
+{
+    switch (c) {
+        case SG_NONE:       return "none";
+        case SG_TERRAIN:    return "terrain";
+        case SG_ASTRO:      return "astro";
+        case SG_FLIGHT:     return "flight";
+        case SG_INPUT:      return "input";
+        case SG_GL:         return "opengl";
+        case SG_VIEW:       return "view";
+        case SG_COCKPIT:    return "cockpit";
+        case SG_GENERAL:    return "general";
+        case SG_MATH:       return "math";
+        case SG_EVENT:      return "event";
+        case SG_AIRCRAFT:   return "aircraft";
+        case SG_AUTOPILOT:  return "autopilot";
+        case SG_IO:         return "io";
+        case SG_CLIPPER:    return "clipper";
+        case SG_NETWORK:    return "network";
+        case SG_ATC:        return "atc";
+        case SG_NASAL:      return "nasal";
+        case SG_INSTR:      return "instruments";
+        case SG_SYSTEMS:    return "systems";
+        case SG_AI:         return "ai";
+        case SG_ENVIRONMENT:return "environment";
+        case SG_SOUND:      return "sound";
+        case SG_NAVAID:     return "navaid";
+        case SG_GUI:        return "gui";
+        case SG_TERRASYNC:  return "terrasync";
+        case SG_PARTICLES:  return "particles";
+        default:            return "unknown";
+    }
+}
+
 } // of namespace simgear
 
 //////////////////////////////////////////////////////////////////////////////
@@ -177,13 +179,13 @@ public:
 
 #endif
 
-class LogStreamPrivate : public SGThread
+class logstream::LogStreamPrivate : public SGThread
 {
 private:
     /**
-     * storage of a single log entry. Note this is not used for a persistent
-     * store, but rather for short term buffering between the submitting
-     * and output threads.
+     * storage of a single log entry. This is used to pass log entries from
+     * the various threads to the logging thread, and also to store the startup
+     * entries
      */
     class LogEntry
     {
@@ -195,19 +197,24 @@ private:
         {
         }
 
-        sgDebugClass debugClass;
-        sgDebugPriority debugPriority;
+        const sgDebugClass debugClass;
+        const sgDebugPriority debugPriority;
         const char* file;
-        int line;
-        std::string message;
+        const int line;
+        const std::string message;
     };
 
+    /**
+     * RAII object to pause the logging thread if it's running, and restart it.
+     * used to safely make configuration changes.
+     */
     class PauseThread
     {
     public:
-        PauseThread(LogStreamPrivate* parent) : m_parent(parent)
+        PauseThread(LogStreamPrivate* parent)
+            : m_parent(parent)
+            , m_wasRunning(m_parent->stop())
         {
-            m_wasRunning = m_parent->stop();
         }
 
         ~PauseThread()
@@ -218,17 +225,13 @@ private:
         }
     private:
         LogStreamPrivate* m_parent;
-        bool m_wasRunning;
+        const bool m_wasRunning;
     };
+
 public:
     LogStreamPrivate() :
         m_logClass(SG_ALL),
-        m_logPriority(SG_ALERT),
-#if defined (SG_WINDOWS)
-        m_stdout_isRedirectedAlready(false),
-        m_stderr_isRedirectedAlready(false),
-#endif
-        m_isRunning(false)
+        m_logPriority(SG_ALERT)
     {
 #if defined (SG_WINDOWS)
         /*
@@ -332,7 +335,7 @@ public:
 
     ~LogStreamPrivate()
     {
-        BOOST_FOREACH(simgear::LogCallback* cb, m_callbacks) {
+        for (simgear::LogCallback* cb : m_callbacks) {
             delete cb;
         }
     }
@@ -340,6 +343,10 @@ public:
     SGMutex m_lock;
     SGBlockingQueue<LogEntry> m_entries;
 
+    // log entries posted during startup
+    std::vector<LogEntry> m_startupEntries;
+    bool m_startupLogging = false;
+
     typedef std::vector<simgear::LogCallback*> CallbackVec;
     CallbackVec m_callbacks;
     /// subset of callbacks which correspond to stdout / console,
@@ -348,12 +355,13 @@ public:
 
     sgDebugClass m_logClass;
     sgDebugPriority m_logPriority;
-    bool m_isRunning;
+    bool m_isRunning = false;
 #if defined (SG_WINDOWS)
     // track whether the console was redirected on launch (in the constructor, which is called early on)
-    bool m_stderr_isRedirectedAlready;
-    bool m_stdout_isRedirectedAlready;
+    bool m_stderr_isRedirectedAlready = false;
+    bool m_stdout_isRedirectedAlready = false;
 #endif
+    bool m_developerMode = false;
 
     void startLog()
     {
@@ -363,6 +371,16 @@ public:
         start();
     }
 
+    void setStartupLoggingEnabled(bool on)
+    {
+        if (m_startupLogging == on) {
+            return;
+        }
+
+        m_startupLogging = on;
+        m_startupEntries.clear();
+    }
+
     virtual void run()
     {
         while (1) {
@@ -373,8 +391,14 @@ public:
                 return;
             }
 
+            if (m_startupLogging) {
+                // save to the startup list for not-yet-added callbacks to
+                // pull down on startup
+                m_startupEntries.push_back(entry);
+            }
+
             // submit to each installed callback in turn
-            BOOST_FOREACH(simgear::LogCallback* cb, m_callbacks) {
+            for (simgear::LogCallback* cb : m_callbacks) {
                 (*cb)(entry.debugClass, entry.debugPriority,
                     entry.file, entry.line, entry.message);
             }
@@ -401,6 +425,13 @@ public:
     {
         PauseThread pause(this);
         m_callbacks.push_back(cb);
+
+        // we clear startup entries not using this, so always safe to run
+        // this code, container will simply be empty
+        for (auto entry : m_startupEntries) {
+            (*cb)(entry.debugClass, entry.debugPriority,
+               entry.file, entry.line, entry.message);
+        }
     }
 
     void removeCallback(simgear::LogCallback* cb)
@@ -424,6 +455,7 @@ public:
 
     bool would_log( sgDebugClass c, sgDebugPriority p ) const
     {
+        p = translatePriority(p);
         if (p >= SG_INFO) return true;
         return ((c & m_logClass) != 0 && p >= m_logPriority);
     }
@@ -431,53 +463,126 @@ public:
     void log( sgDebugClass c, sgDebugPriority p,
             const char* fileName, int line, const std::string& msg)
     {
+        p = translatePriority(p);
         LogEntry entry(c, p, fileName, line, msg);
         m_entries.push(entry);
     }
+
+    sgDebugPriority translatePriority(sgDebugPriority in) const
+    {
+        if (in == SG_DEV_WARN) {
+            return m_developerMode ? SG_WARN : SG_DEBUG;
+        }
+
+        if (in == SG_DEV_ALERT) {
+            return m_developerMode ? SG_POPUP : SG_WARN;
+        }
+
+        return in;
+    }
 };
 
 /////////////////////////////////////////////////////////////////////////////
 
-static logstream* global_logstream = NULL;
-static LogStreamPrivate* global_privateLogstream = NULL;
+static std::unique_ptr<logstream> global_logstream;
 static SGMutex global_logStreamLock;
 
 logstream::logstream()
 {
-    global_privateLogstream = new LogStreamPrivate;
-    global_privateLogstream->startLog();
+    d.reset(new LogStreamPrivate);
+    d->startLog();
 }
 
 logstream::~logstream()
 {
     popup_msgs.clear();
-    global_privateLogstream->stop();
-    delete global_privateLogstream;
+    d->stop();
 }
 
 void
 logstream::setLogLevels( sgDebugClass c, sgDebugPriority p )
 {
-    global_privateLogstream->setLogLevels(c, p);
+    d->setLogLevels(c, p);
 }
 
+void logstream::setDeveloperMode(bool devMode)
+{
+    d->m_developerMode = devMode;
+}
+
+
 void
 logstream::addCallback(simgear::LogCallback* cb)
 {
-    global_privateLogstream->addCallback(cb);
+    d->addCallback(cb);
 }
 
 void
 logstream::removeCallback(simgear::LogCallback* cb)
 {
-    global_privateLogstream->removeCallback(cb);
+    d->removeCallback(cb);
 }
 
 void
 logstream::log( sgDebugClass c, sgDebugPriority p,
         const char* fileName, int line, const std::string& msg)
 {
-    global_privateLogstream->log(c, p, fileName, line, msg);
+    d->log(c, p, fileName, line, msg);
+}
+
+
+void logstream::hexdump(sgDebugClass c, sgDebugPriority p, const char* fileName, int line, const void *mem, unsigned int len, int columns)
+{
+    unsigned int i, j;
+    char temp[3000], temp1[3000];
+    *temp = 0;
+
+    for (i = 0; i < len + ((len % columns) ? (columns - len % columns) : 0); i++)
+    {
+        if (strlen(temp) > 500) return;
+        /* print offset */
+        if (i % columns == 0)
+        {
+            sprintf(temp1, "0x%06x: ", i);
+            strcat(temp, temp1);
+        }
+
+        /* print hex data */
+        if (i < len)
+        {
+            sprintf(temp1, "%02x ", 0xFF & ((char*)mem)[i]);
+            strcat(temp, temp1);
+        }
+        else /* end of block, just aligning for ASCII dump */
+        {
+            strcat(temp, "   ");
+        }
+
+        /* print ASCII dump */
+        if (i % columns == (columns - 1))
+        {
+            for (j = i - (columns - 1); j <= i; j++)
+            {
+                if (j >= len) /* end of block, not really printing */
+                {
+                    strcat(temp, " ");
+                }
+                else if (((((char*)mem)[j]) & (char)0x7f) > 32) /* printable char */
+                {
+                    char t2[2];
+                    t2[0] = 0xFF & ((char*)mem)[j];
+                    t2[1] = 0;
+                    strcat(temp, t2);
+                }
+                else /* other char */
+                {
+                    strcat(temp, ".");
+                }
+            }
+            log(c, p, fileName, line, temp );
+            *temp = 0;
+        }
+    }
 }
 
 void
@@ -507,31 +612,31 @@ logstream::has_popup()
 bool
 logstream::would_log( sgDebugClass c, sgDebugPriority p ) const
 {
-    return global_privateLogstream->would_log(c,p);
+    return d->would_log(c,p);
 }
 
 sgDebugClass
 logstream::get_log_classes() const
 {
-    return global_privateLogstream->m_logClass;
+    return d->m_logClass;
 }
 
 sgDebugPriority
 logstream::get_log_priority() const
 {
-    return global_privateLogstream->m_logPriority;
+    return d->m_logPriority;
 }
 
 void
 logstream::set_log_priority( sgDebugPriority p)
 {
-    global_privateLogstream->setLogLevels(global_privateLogstream->m_logClass, p);
+    d->setLogLevels(d->m_logClass, p);
 }
 
 void
 logstream::set_log_classes( sgDebugClass c)
 {
-    global_privateLogstream->setLogLevels(c, global_privateLogstream->m_logPriority);
+    d->setLogLevels(c, d->m_logPriority);
 }
 
 
@@ -547,49 +652,54 @@ sglog()
     SGGuard<SGMutex> g(global_logStreamLock);
 
     if( !global_logstream )
-        global_logstream = new logstream();
-    return *global_logstream;
+        global_logstream.reset(new logstream);
+    return *(global_logstream.get());
 }
 
 void
 logstream::logToFile( const SGPath& aPath, sgDebugClass c, sgDebugPriority p )
 {
-    global_privateLogstream->addCallback(new FileLogCallback(aPath, c, p));
+    d->addCallback(new FileLogCallback(aPath, c, p));
 }
 
-namespace simgear
+void logstream::setStartupLoggingEnabled(bool enabled)
 {
+    d->setStartupLoggingEnabled(enabled);
+}
 
-void requestConsole()
-{ 
+void logstream::requestConsole()
+{
 #if defined (SG_WINDOWS)
-   /*
-    * 2016-09-20(RJH) - Reworked console handling
-    * This is part of the reworked console handling for Win32. This is for building as a Win32 GUI Subsystem where no
-    * console is allocated on launch. If building as a console app then the startup will ensure that a console is created - but
-    * we don't need to handle that.
-    * The new handling is quite simple:
-    *              1. The constructor will ensure that these streams exists. It will attach to the
-    *                 parent command prompt if started from the command prompt, otherwise the
-    *                 stdout/stderr will be bound to the NUL device.
-    *              2. with --console a window will always appear regardless of where the process was
-    *                 started from. Any non redirected streams will be redirected 
-    *              3. You cannot use --console and either redirected stream.
-    *
-    * This is called after the Private Log Stream constructor so we need to undo any console that it has attached to.
-    */
+    const bool stderrAlreadyRedirected = d->m_stderr_isRedirectedAlready;
+    const bool stdoutAlreadyRedirected = d->m_stdout_isRedirectedAlready;
 
-    if (!global_privateLogstream->m_stderr_isRedirectedAlready && !global_privateLogstream->m_stdout_isRedirectedAlready) {
+    /*
+     * 2016-09-20(RJH) - Reworked console handling
+     * This is part of the reworked console handling for Win32. This is for building as a Win32 GUI Subsystem where no
+     * console is allocated on launch. If building as a console app then the startup will ensure that a console is created - but
+     * we don't need to handle that.
+     * The new handling is quite simple:
+     *              1. The constructor will ensure that these streams exists. It will attach to the
+     *                 parent command prompt if started from the command prompt, otherwise the
+     *                 stdout/stderr will be bound to the NUL device.
+     *              2. with --console a window will always appear regardless of where the process was
+     *                 started from. Any non redirected streams will be redirected
+     *              3. You cannot use --console and either redirected stream.
+     *
+     * This is called after the Private Log Stream constructor so we need to undo any console that it has attached to.
+     */
+
+    if (!stderrAlreadyRedirected && !stdoutAlreadyRedirected) {
         FreeConsole();
         if (AllocConsole()) {
-            if (!global_privateLogstream->m_stdout_isRedirectedAlready)
+            if (!stdoutAlreadyRedirected)
                 freopen("conout$", "w", stdout);
 
-            if (!global_privateLogstream->m_stderr_isRedirectedAlready)
+            if (!stderrAlreadyRedirected)
                 freopen("conout$", "w", stderr);
 
             //http://stackoverflow.com/a/25927081
-            //Clear the error state for each of the C++ standard stream objects. 
+            //Clear the error state for each of the C++ standard stream objects.
             std::wcout.clear();
             std::cout.clear();
             std::wcerr.clear();
@@ -602,11 +712,19 @@ void requestConsole()
 }
 
 
+namespace simgear
+{
+
+void requestConsole()
+{ 
+    sglog().requestConsole();
+}
+
+
 void shutdownLogging()
 {
     SGGuard<SGMutex> g(global_logStreamLock);
-    delete global_logstream;
-    global_logstream = 0;
+    global_logstream.reset();
 }
 
 } // of namespace simgear
