@@ -21,8 +21,9 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 //
 
+#include <simgear_config.h>
 #include "DNSClient.hxx"
-#include "udns.h"
+#include <udns.h>
 #include <time.h>
 #include <simgear/debug/logstream.hxx>
 
@@ -33,18 +34,33 @@ namespace DNS {
 class Client::ClientPrivate {
 public:
     ClientPrivate() {
-        if (dns_init(NULL, 0) < 0)
-            SG_LOG(SG_IO, SG_ALERT, "Can't init udns library" );
 
-        if( dns_open(NULL) < 0 )
+        if( instanceCounter++ == 0 )
+          if (dns_init(NULL, 0) < 0)
+              SG_LOG(SG_IO, SG_ALERT, "Can't init udns library" );
+
+        ctx = dns_new(NULL);
+
+        if (dns_init(ctx, 0) < 0)
+            SG_LOG(SG_IO, SG_ALERT, "Can't create udns context" );
+
+        if( dns_open(ctx) < 0 )
             SG_LOG(SG_IO, SG_ALERT, "Can't open udns context" );
     }
 
     ~ClientPrivate() {
-        dns_close(NULL);
+        dns_close(ctx);
+        dns_free(ctx);
+        if( --instanceCounter == 0 )
+            dns_close(NULL);
     }
+
+    struct dns_ctx * ctx;
+    static size_t instanceCounter;
 };
 
+size_t Client::ClientPrivate::instanceCounter = 0;
+
 Request::Request( const std::string & dn ) :
         _dn(dn),
         _type(DNS_T_ANY),
@@ -111,10 +127,10 @@ static void dnscbSRV(struct dns_ctx *ctx, struct dns_rr_srv *result, void *data)
     r->setComplete();
 }
 
-void SRVRequest::submit()
+void SRVRequest::submit( Client * client )
 {
     // if service is defined, pass service and protocol
-    if (!dns_submit_srv(NULL, getDn().c_str(), _service.empty() ? NULL : _service.c_str(), _service.empty() ? NULL : _protocol.c_str(), 0, dnscbSRV, this )) {
+    if (!dns_submit_srv(client->d->ctx, getDn().c_str(), _service.empty() ? NULL : _service.c_str(), _service.empty() ? NULL : _protocol.c_str(), 0, dnscbSRV, this )) {
         SG_LOG(SG_IO, SG_ALERT, "Can't submit dns request for " << getDn());
         return;
     }
@@ -148,10 +164,10 @@ static void dnscbTXT(struct dns_ctx *ctx, struct dns_rr_txt *result, void *data)
     r->setComplete();
 }
 
-void TXTRequest::submit()
+void TXTRequest::submit( Client * client )
 {
     // protocol and service an already encoded in DN so pass in NULL for both
-    if (!dns_submit_txt(NULL, getDn().c_str(), DNS_C_IN, 0, dnscbTXT, this )) {
+    if (!dns_submit_txt(client->d->ctx, getDn().c_str(), DNS_C_IN, 0, dnscbTXT, this )) {
         SG_LOG(SG_IO, SG_ALERT, "Can't submit dns request for " << getDn());
         return;
     }
@@ -175,11 +191,11 @@ static void dnscbNAPTR(struct dns_ctx *ctx, struct dns_rr_naptr *result, void *d
         r->ttl = result->dnsnaptr_ttl;
         for (int i = 0; i < result->dnsnaptr_nrr; i++) {
             if( !r->qservice.empty() && r->qservice != result->dnsnaptr_naptr[i].service )
-                return;
+                continue;
 
             //TODO: case ignore and result flags may have more than one flag
             if( !r->qflags.empty() && r->qflags != result->dnsnaptr_naptr[i].flags )
-                return;
+                continue;
 
             NAPTRRequest::NAPTR_ptr naptr(new NAPTRRequest::NAPTR);
             r->entries.push_back(naptr);
@@ -196,9 +212,9 @@ static void dnscbNAPTR(struct dns_ctx *ctx, struct dns_rr_naptr *result, void *d
     r->setComplete();
 }
 
-void NAPTRRequest::submit()
+void NAPTRRequest::submit( Client * client )
 {
-    if (!dns_submit_naptr(NULL, getDn().c_str(), 0, dnscbNAPTR, this )) {
+    if (!dns_submit_naptr(client->d->ctx, getDn().c_str(), 0, dnscbNAPTR, this )) {
         SG_LOG(SG_IO, SG_ALERT, "Can't submit dns request for " << getDn());
         return;
     }
@@ -217,15 +233,16 @@ Client::Client() :
 
 void Client::makeRequest(const Request_ptr& r)
 {
-    r->submit();
+    r->submit(this);
 }
 
 void Client::update(int waitTimeout)
 {
     time_t now = time(NULL);
-    if( dns_timeouts( NULL, waitTimeout, now ) < 0 )
+    if( dns_timeouts( d->ctx, -1, now ) < 0 )
         return;
-    dns_ioevent(NULL, now);
+
+    dns_ioevent(d->ctx, now);
 }
 
 } // of namespace DNS
