@@ -27,7 +27,7 @@
 #include <simgear_config.h>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/misc/strutils.hxx>
-#include <simgear/misc/sgstream.hxx>
+#include <simgear/io/iostreams/sgstream.hxx>
 
 #include <stdio.h>
 #include <sys/stat.h>
@@ -53,9 +53,9 @@ static const char sgDirPathSep = '/';
 static const char sgDirPathSepBad = '\\';
 
 #ifdef _WIN32
-const char SGPath::pathListSep = ';';
+const char SGPath::pathListSep[] = ";"; // this is null-terminated
 #else
-const char SGPath::pathListSep = ':';
+const char SGPath::pathListSep[] = ":"; // ditto
 #endif
 
 #ifdef _WIN32
@@ -278,7 +278,6 @@ SGPath& SGPath::operator=(const SGPath& p)
 SGPath::~SGPath() {
 }
 
-#if defined(ENABLE_OLD_PATH_API)
 // set path
 void SGPath::set( const string& p ) {
     path = p;
@@ -286,7 +285,6 @@ void SGPath::set( const string& p ) {
     _cached = false;
     _rwCached = false;
 }
-#endif
 
 //------------------------------------------------------------------------------
 void SGPath::setPermissionChecker(PermissionChecker validator)
@@ -334,7 +332,7 @@ SGPath SGPath::operator/( const std::string& p ) const
 #if defined(ENABLE_OLD_PATH_API)
 //add a new path component to the existing path string
 void SGPath::add( const string& p ) {
-    append( SGPath::pathListSep+p );
+    append( SGPath::pathListSep[0] + p );
 }
 #endif
 
@@ -460,10 +458,11 @@ void SGPath::validate() const
 
   if (path.empty()) {
 	  _exists = false;
+      _canWrite = _canRead = false;
 	  return;
   }
 
-#ifdef _WIN32
+#if defined(SG_WINDOWS)
   struct _stat buf ;
   bool remove_trailing = false;
   std::wstring statPath(wstr());
@@ -473,12 +472,24 @@ void SGPath::validate() const
 
   if (_wstat(statPath.c_str(), &buf ) < 0) {
     _exists = false;
+    _canRead = false;
+
+      // check parent directory for write-ability
+      std::wstring parentPath = simgear::strutils::convertUtf8ToWString(dir());
+      struct _stat parentBuf;
+      if (_wstat(parentPath.c_str(), &parentBuf) >= 0) {
+          _canWrite = parentBuf.st_mode & _S_IWRITE;
+      } else {
+          _canWrite = false;
+      }
   } else {
     _exists = true;
     _isFile = ((S_IFREG & buf.st_mode ) !=0);
     _isDir = ((S_IFDIR & buf.st_mode ) !=0);
     _modTime = buf.st_mtime;
     _size = buf.st_size;
+	_canRead = _S_IREAD & buf.st_mode;
+	_canWrite = _S_IWRITE & buf.st_mode;
   }
 
 #else
@@ -486,36 +497,46 @@ void SGPath::validate() const
 
   if (stat(path.c_str(), &buf ) < 0) {
     _exists = false;
+    _canRead = false;
+
+      // check parent directory for write-ability
+      std::string parentPath = dir();
+      struct stat parentBuf;
+      if (stat(parentPath.c_str(), &parentBuf) >= 0) {
+          _canWrite = parentBuf.st_mode & S_IWUSR;
+      } else {
+          _canWrite = false;
+      }
   } else {
     _exists = true;
     _isFile = ((S_ISREG(buf.st_mode )) != 0);
     _isDir = ((S_ISDIR(buf.st_mode )) != 0);
     _modTime = buf.st_mtime;
     _size = buf.st_size;
+    _canRead = S_IRUSR & buf.st_mode;
+    _canWrite = S_IWUSR & buf.st_mode;
   }
 
 #endif
+    // ensure permissions are no less restrictive than what the
+    // permissions checker offers
+    if ( _permission_checker ) {
+        Permissions p = _permission_checker(*this);
+        _canRead &= p.read;
+        _canWrite &= p.write;
+    }
+
   _cached = true;
 }
 
 //------------------------------------------------------------------------------
 void SGPath::checkAccess() const
 {
-  if( _rwCached && _cacheEnabled )
+  if ( _rwCached && _cacheEnabled ) {
     return;
-
-  if( _permission_checker )
-  {
-    Permissions p = _permission_checker(*this);
-    _canRead = p.read;
-    _canWrite = p.write;
-  }
-  else
-  {
-    _canRead = true;
-    _canWrite = true;
   }
 
+  validate();
   _rwCached = true;
 }
 
@@ -555,7 +576,7 @@ bool SGPath::isFile() const
 
 int SGPath::create_dir(mode_t mode)
 {
-  if( !canWrite() )
+  if ( !permissionsAllowsWrite() )
   {
     SG_LOG( SG_IO,
             SG_WARN, "Error creating directory for '" << *this << "'"
@@ -644,7 +665,7 @@ string_list sgPathSplit( const string &search_path ) {
     bool done = false;
 
     while ( !done ) {
-        int index = tmp.find(SGPath::pathListSep);
+        int index = tmp.find(SGPath::pathListSep[0]);
         if (index >= 0) {
             result.push_back( tmp.substr(0, index) );
             tmp = tmp.substr( index + 1 );
@@ -703,7 +724,7 @@ std::string SGPath::str_native() const
 //------------------------------------------------------------------------------
 bool SGPath::remove()
 {
-  if( !canWrite() )
+  if( !permissionsAllowsWrite() )
   {
     SG_LOG( SG_IO, SG_WARN, "file remove failed: (" << *this << ")"
                                                " reason: access denied" );
@@ -712,7 +733,15 @@ bool SGPath::remove()
 
 #if defined(SG_WINDOWS)
   std::wstring ps = wstr();
-  int err = _wunlink(ps.c_str());
+
+  // windows forbids removing a read-only file, let's try to deal
+  // with that case
+  int err = _wchmod(ps.c_str(), _S_IWRITE | _S_IREAD);
+  if (err != 0) {
+	  SG_LOG(SG_IO, SG_WARN, "failed to make file writeable prior to remove:" << *this);
+  } else {
+	  err = _wunlink(ps.c_str());
+  }
 #else
   std::string ps = local8BitStr();
   int err = ::unlink(ps.c_str());
@@ -1018,3 +1047,10 @@ std::wstring SGPath::wstr() const
 {
 	return simgear::strutils::convertUtf8ToWString(path);
 }
+
+//------------------------------------------------------------------------------
+bool SGPath::permissionsAllowsWrite() const
+{
+    return _permission_checker ? _permission_checker(*this).write : true;
+}
+
