@@ -234,6 +234,7 @@ SGSubsystemGroup::update (double delta_time_sec)
     if ((delta_time_sec > 0.0) && (_fixedUpdateTime > 0.0)) {
       double localDelta = delta_time_sec + _updateTimeRemainder;
       loopCount = SGMiscd::roundToInt(localDelta / _fixedUpdateTime);
+      loopCount = std::max(0, loopCount);
       _updateTimeRemainder = delta_time_sec - (loopCount * _fixedUpdateTime);
       delta_time_sec = _fixedUpdateTime;
     }
@@ -301,8 +302,6 @@ SGSubsystemGroup::set_subsystem (const string &name, SGSubsystem * subsystem,
                                  double min_step_sec)
 {
     Member * member = get_member(name, true);
-    if (member->subsystem != 0)
-        delete member->subsystem;
     member->name = name;
     member->subsystem = subsystem;
     member->min_step_sec = min_step_sec;
@@ -430,11 +429,11 @@ SGSubsystemGroup::Member::update (double delta_time_sec)
 
 
 SGSubsystemMgr::SGSubsystemMgr () :
+  _groups(MAX_GROUPS),
   _initPosition(0)
 {
-  for (int i = 0; i < MAX_GROUPS; i++) {
-    _groups[i] = new SGSubsystemGroup;
-  }
+  for (int i = 0; i < MAX_GROUPS; i++)
+    _groups[i].reset(new SGSubsystemGroup);
 }
 
 SGSubsystemMgr::~SGSubsystemMgr ()
@@ -442,10 +441,7 @@ SGSubsystemMgr::~SGSubsystemMgr ()
   // ensure get_subsystem returns NULL from now onwards,
   // before the SGSubsystemGroup destructors are run
   _subsystem_map.clear();
-  
-  for (int i = 0; i < MAX_GROUPS; i++) {
-    delete _groups[i];
-  }
+  _groups.clear();
 }
 
 void
